@@ -1,9 +1,12 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 try:
-    import PAM
+    import pam
+    print("Using pam module (python3-pampy)")
 except:
-    import pam as PAM
+    import PAM
+    print("Using PAM module (python3-pam)")
+    pam = None
 import pexpect
 import time
 from random import randint
@@ -11,12 +14,18 @@ import shutil
 import os
 import subprocess
 
-import PIL
+from PIL import Image
 import gi
 gi.require_version('AccountsService', '1.0')
-from gi.repository import AccountsService, GLib
+from gi.repository import AccountsService, GLib, GdkPixbuf
 
-from GSettingsWidgets import *
+from SettingsWidgets import SidePage
+from ChooserButtonWidgets import PictureChooserButton
+from xapp.GSettingsWidgets import *
+
+class PasswordError(Exception):
+    """Exception raised when an incorrect password is supplied."""
+    pass
 
 
 class Module:
@@ -25,20 +34,26 @@ class Module:
     comment = _("Change your user preferences and password")
 
     def __init__(self, content_box):
-        keywords = _("user, account, information, details")
+        keywords = _("user, account, information, details, password")
         sidePage = SidePage(_("Account details"), "cs-user", keywords, content_box, module=self)
         self.sidePage = sidePage
+        self.window = None
+
+    def _setParentRef(self, window):
+        self.window = window
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading User module"
+            print("Loading User module")
 
             page = SettingsPage()
             self.sidePage.add_widget(page)
 
             settings = page.add_section(_("Account details"))
 
-            self.face_button = PictureChooserButton(num_cols=4, button_picture_size=64, menu_pictures_size=64, keep_square=True)
+            self.scale = self.window.get_scale_factor()
+
+            self.face_button = PictureChooserButton(num_cols=4, button_picture_size=64, menu_pictures_size=64*self.scale, keep_square=True)
             self.face_button.set_alignment(0.0, 0.5)
             self.face_button.set_tooltip_text(_("Click to change your picture"))
 
@@ -77,7 +92,7 @@ class Module:
             widget = SettingsWidget()
             label = Gtk.Label.new(_("Password"))
             widget.pack_start(label, False, False, 0)
-            password_mask = Gtk.Label.new(u'\u2022\u2022\u2022\u2022\u2022\u2022')
+            password_mask = Gtk.Label.new('\u2022\u2022\u2022\u2022\u2022\u2022')
             password_mask.set_alignment(0.9, 0.5)
             self.password_button = Gtk.Button()
             size_group.add_widget(self.password_button)
@@ -124,14 +139,14 @@ class Module:
 
         # streamer takes -t photos, uses /dev/video0
         if 0 != subprocess.call(["streamer", "-j90", "-t8", "-s800x600", "-o", "/tmp/temp-account-pic00.jpeg"]):
-            print "Error: Webcam not available"
+            print("Error: Webcam not available")
             return
 
         # Use the 8th frame (the webcam takes a few frames to "lighten up")
         path = "/tmp/temp-account-pic07.jpeg"
 
         # Crop the image to thumbnail size
-        image = PIL.Image.open(path)
+        image = Image.open(path)
         width, height = image.size
 
         if width > height:
@@ -150,7 +165,7 @@ class Module:
         bottom = (height + new_height) / 2
 
         image = image.crop((left, top, right, bottom))
-        image.thumbnail((255, 255), PIL.Image.ANTIALIAS)
+        image.thumbnail((255, 255), Image.LANCZOS)
 
         face_path = os.path.join(self.accountService.get_home_dir(), ".face")
 
@@ -186,8 +201,8 @@ class Module:
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
             path = dialog.get_filename()
-            image = PIL.Image.open(path)
-            image.thumbnail((255, 255), PIL.Image.ANTIALIAS)
+            image = Image.open(path)
+            image.thumbnail((255, 255), Image.LANCZOS)
             face_path = os.path.join(self.accountService.get_home_dir(), ".face")
             image.save(face_path, "png")
             self.accountService.set_icon_file(face_path)
@@ -253,6 +268,7 @@ class PasswordDialog(Gtk.Dialog):
         self.new_password = Gtk.Entry()
         self.new_password.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "view-refresh")
         self.new_password.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, _("Generate a password"))
+        self.new_password.set_tooltip_text(_("Generate a password"))
         self.new_password.connect("icon-release", self._on_new_password_icon_released)
         self.new_password.connect("changed", self._on_passwords_changed)
         table.attach(self.new_password, 1, 3, 1, 2)
@@ -317,7 +333,7 @@ class PasswordDialog(Gtk.Dialog):
         passwd.close()
 
         if passwd.exitstatus is None or passwd.exitstatus > 0:
-            self.infobar.show()
+            self.infobar.show_all()
         else:
             self.destroy()
 
@@ -342,24 +358,41 @@ class PasswordDialog(Gtk.Dialog):
     def _on_show_password_toggled(self, widget):
         self.set_passwords_visibility()
 
+    def auth_pam(self):
+        if not pam.pam().authenticate(GLib.get_user_name(), self.current_password.get_text(), 'passwd'):
+            raise PasswordError("Invalid password")
+
+    def auth_PyPAM(self):
+        auth = PAM.pam()
+        auth.start('passwd')
+        auth.set_item(PAM.PAM_USER, GLib.get_user_name())
+        auth.set_item(PAM.PAM_CONV, self.pam_conv)
+        try:
+            auth.authenticate()
+            auth.acct_mgmt()
+            return True
+        except PAM.error as resp:
+            raise PasswordError("Invalid password")
+
     def _on_current_password_changed(self, widget, event):
         self.infobar.hide()
         if self.current_password.get_text() != "":
-            auth = PAM.pam()
-            auth.start('passwd')
-            auth.set_item(PAM.PAM_USER, GLib.get_user_name())
-            auth.set_item(PAM.PAM_CONV, self.pam_conv)
             try:
-                auth.authenticate()
-                auth.acct_mgmt()
-            except PAM.error, resp:
+                self.auth_pam() if pam else self.auth_PyPAM()
+            except PasswordError:
                 self.current_password.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_DIALOG_WARNING)
                 self.current_password.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, _("Wrong password"))
+                self.current_password.set_tooltip_text(_("Wrong password"))
                 self.correct_current_password = False
             except:
-                print 'Internal error'
+                self.current_password.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_DIALOG_WARNING)
+                self.current_password.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, _("Internal Error"))
+                self.current_password.set_tooltip_text(_("Internal Error"))
+                self.correct_current_password = False
+                raise
             else:
                 self.current_password.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, None)
+                self.current_password.set_tooltip_text("")
                 self.correct_current_password = True
                 self.check_passwords()
 
@@ -401,8 +434,10 @@ class PasswordDialog(Gtk.Dialog):
         if new_password != confirm_password:
             self.confirm_password.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_DIALOG_WARNING)
             self.confirm_password.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, _("Passwords do not match"))
+            self.confirm_password.set_tooltip_text(_("Passwords do not match"))
         else:
             self.confirm_password.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, None)
+            self.confirm_password.set_tooltip_text("")
         if len(new_password) < 8:
             self.strengh_label.set_text(_("Too short"))
             self.strengh_indicator.set_fraction(0.0)
