@@ -36,12 +36,16 @@ var deskletChangeKey = 0;
 const ENABLED_DESKLETS_KEY = 'enabled-desklets';
 const DESKLET_SNAP_KEY = 'desklet-snap';
 const DESKLET_SNAP_INTERVAL_KEY = 'desklet-snap-interval';
+const KEYBINDING_SCHEMA = 'org.cinnamon.desktop.keybindings';
+const SHOW_DESKLETS_KEY = 'show-desklets';
+const LOCK_DESKLETS_KEY = "lock-desklets";
 
 function initEnabledDesklets() {
     for (let i = 0; i < definitions.length; i++) {
         promises.push(Extension.loadExtension(definitions[i].uuid, Extension.Type.DESKLET))
     }
     return Promise.all(promises).then(function() {
+        Main.cinnamonDBusService.EmitXletsLoadedComplete();
         promises = [];
     });
 }
@@ -57,15 +61,15 @@ function unloadRemovedDesklets(removedDeskletUUIDs) {
  *
  * Initialize desklet manager
  */
-function init(){
-    let startTime = new Date().getTime();
+function init() {
+    const startTime = new Date().getTime();
     try {
         desklets = imports.desklets;
     } catch (e) {
         desklets = {};
     }
     deskletMeta = Extension.Type.DESKLET.legacyMeta;
-    deskletsLoaded = false
+    deskletsLoaded = false;
 
     definitions = getDefinitions();
 
@@ -75,7 +79,7 @@ function init(){
         global.settings.connect('changed::' + DESKLET_SNAP_INTERVAL_KEY, _onDeskletSnapChanged);
 
         deskletsLoaded = true;
-        enableMouseTracking(true);
+        updateMouseTracking();
         global.log(`DeskletManager started in ${new Date().getTime() - startTime} ms`);
     });
 }
@@ -84,7 +88,8 @@ function getDeskletDefinition(definition) {
     return queryCollection(definitions, definition);
 }
 
-function enableMouseTracking(enable) {
+function updateMouseTracking() {
+    let enable = definitions.length > 0;
     if (enable && !mouseTrackTimoutId) {
         mouseTrackTimoutId = Mainloop.timeout_add(500, checkMouseTracking);
     } else if (!enable && mouseTrackTimoutId) {
@@ -100,7 +105,7 @@ function enableMouseTracking(enable) {
 }
 
 function hasMouseWindow(){
-    let window = global.screen.get_mouse_window(null);
+    let window = global.display.get_pointer_window(null);
     return window && window.window_type !== Meta.WindowType.DESKTOP;
 }
 
@@ -190,6 +195,19 @@ function prepareExtensionUnload(extension, deleteConfig) {
     }
 }
 
+// Callback for extension.js
+function prepareExtensionReload(extension) {
+    for (var i = 0; i < definitions.length; i++) {
+        if (extension.uuid === definitions[i].uuid) {
+            let {desklet, desklet_id} = definitions[i];
+            if (!desklet) continue;
+            global.log("Reloading desklet: " + extension.uuid + "/" + desklet_id);
+            desklet.on_desklet_reloaded();
+            return;
+        }
+    }
+}
+
 function _onEnabledDeskletsChanged() {
     let oldDefinitions = definitions.slice();
     definitions = getDefinitions();
@@ -237,7 +255,7 @@ function _onEnabledDeskletsChanged() {
 
     // Make sure all desklet extensions are loaded.
     // Once loaded, the desklets will add themselves via finishExtensionLoad
-    initEnabledDesklets();
+    initEnabledDesklets().then(updateMouseTracking);
 }
 
 function _unloadDesklet(deskletDefinition, deleteConfig) {
@@ -258,17 +276,20 @@ function _unloadDesklet(deskletDefinition, deleteConfig) {
 }
 
 function _removeDeskletConfigFile(uuid, instanceId) {
-    let config_path = (GLib.get_home_dir() + "/" +
-                               ".cinnamon" + "/" +
-                                 "configs" + "/" +
-                                      uuid + "/" +
-                                instanceId + ".json");
-    let file = Gio.File.new_for_path(config_path);
-    if (file.query_exists(null)) {
-        try {
-            file.delete(null);
-        } catch (e) {
-            global.logError("Problem removing desklet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+    let config_paths = [
+        [GLib.get_home_dir(), ".cinnamon", "configs", uuid, instanceId + ".json"].join("/"),
+        [GLib.get_user_config_dir(), "cinnamon", "spices", uuid, instanceId + ".json"].join("/")
+    ];
+
+    for (let i = 0; i < config_paths.length; i++) {
+        const config_path = config_paths[i];
+        let file = Gio.File.new_for_path(config_path);
+        if (file.query_exists(null)) {
+            try {
+                file.delete(null);
+            } catch (e) {
+                global.logError("Problem removing desklet config file during cleanup.  UUID is " + uuid + " and filename is " + config_path);
+            }
         }
     }
 }
@@ -391,8 +412,9 @@ function _onDeskletSnapChanged(){
 
         enabledDesklets[i] = elements.join(":");
     }
-
+    global.settings.disconnect(deskletChangeKey);
     global.settings.set_strv(ENABLED_DESKLETS_KEY, enabledDesklets);
+    deskletChangeKey = global.settings.connect('changed::' + ENABLED_DESKLETS_KEY, _onEnabledDeskletsChanged);
     return;
 }
 
@@ -407,7 +429,7 @@ function DeskletContainer(){
 
 DeskletContainer.prototype = {
     _init: function(){
-        this.actor = new Clutter.Group();
+        this.actor = global.desklet_container;
         this.actor._delegate = this;
 
         this.last_x = -1;
@@ -418,6 +440,25 @@ DeskletContainer.prototype = {
 
         this.isModal = false;
         this.stageEventIds = [];
+
+        this.keyBindingSettings = new Gio.Settings({ schema_id: KEYBINDING_SCHEMA });
+        this.keyBindingSettings.connect('changed::show-desklets', () => this.applyKeyBindings());
+        this.applyKeyBindings();
+        global.settings.connect('changed::panel-edit-mode', () => {
+            if (this.isModal) {
+                this.lower();
+            }
+        });
+
+        global.settings.connect('changed::' + LOCK_DESKLETS_KEY, () => this.onDeskletsLockedChanged());
+    },
+
+    applyKeyBindings: function() {
+        Main.keybindingManager.addHotKeyArray(
+            SHOW_DESKLETS_KEY,
+            this.keyBindingSettings.get_strv(SHOW_DESKLETS_KEY),
+            () => this.toggle()
+        );
     },
 
     /**
@@ -428,6 +469,7 @@ DeskletContainer.prototype = {
      */
     addDesklet: function(actor){
         this.actor.add_actor(actor);
+        actor._delegate._draggable.inhibit = global.settings.get_boolean(LOCK_DESKLETS_KEY);
     },
 
     /**
@@ -442,6 +484,12 @@ DeskletContainer.prototype = {
         return this.actor.contains(actor);
     },
 
+    onDeskletsLockedChanged: function(settings, key) {
+        this.actor.get_children().forEach((deskletActor) => {
+            deskletActor._delegate._draggable.inhibit = global.settings.get_boolean(LOCK_DESKLETS_KEY);
+        });
+    },
+
     handleDragOver: function(source, actor, x, y, time) {
         deskletsDragging = true;
 
@@ -515,6 +563,8 @@ DeskletContainer.prototype = {
                 if (global.settings.get_boolean(DESKLET_SNAP_KEY)){
                     elements[2] = this._dragPlaceholder.x
                     elements[3] = this._dragPlaceholder.y;
+                    actor.set_x(this._dragPlaceholder.x);
+                    actor.set_y(this._dragPlaceholder.y);
                 }
                 definition = elements.join(":");
                 enabledDesklets[i] = definition;
@@ -581,9 +631,8 @@ DeskletContainer.prototype = {
     handleStageEvent: function(actor, event) {
         let target = event.get_source();
         let type = event.type();
-
         if ((type === Clutter.EventType.BUTTON_PRESS || type === Clutter.EventType.BUTTON_RELEASE)
-            && target.toString().indexOf('ClutterStage') > -1) {
+            && target.get_parent() instanceof Meta.WindowActor) {
             this.lower();
         }
 
@@ -594,12 +643,20 @@ DeskletContainer.prototype = {
         if (this.actor.get_children().length === 0) {
             return;
         }
-        this.actor.get_parent().set_child_above_sibling(this.actor, null);
+        global.display.set_desklets_above(true);
         this.setModal();
     },
 
     lower: function() {
-        this.actor.get_parent().set_child_below_sibling(this.actor, global.window_group);
+        global.display.set_desklets_above(false);
         this.unsetModal();
+    },
+
+    toggle: function() {
+        if (this.isModal) {
+            this.lower();
+        } else {
+            this.raise();
+        }
     }
 };
